@@ -32,6 +32,11 @@ __date__ = 'Feb 7, 2013'
 def pw_check(ids, args):
     if len(ids) > FWConfig().PW_CHECK_NUM:
         m_password = datetime.datetime.now().strftime('%Y-%m-%d')
+        if not args.password:
+            if raw_input('Are you sure? This will modify {} entries. (Y/N)'.format(len(ids)))[0].upper() == 'Y':
+                args.password=datetime.datetime.now().strftime('%Y-%m-%d')
+            else:
+                raise ValueError('Operation aborted by user.')
         if args.password != m_password:
             raise ValueError("Modifying more than {} entries requires setting the --password parameter! (Today's date, e.g. 2012-02-25)".format(FWConfig().PW_CHECK_NUM))
 
@@ -72,8 +77,7 @@ def lpad():
     subparsers = parser.add_subparsers(help='command', dest='command')
 
     reset_parser = subparsers.add_parser('reset', help='reset and re-initialize the FireWorks database')
-    reset_parser.add_argument('password', help="Today's date, e.g. 2012-02-25. Required to prevent \
-    against accidental initializations.")
+    reset_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required to protect against accidental reset.")
 
     addwf_parser = subparsers.add_parser('add', help='insert a Workflow from file')
     addwf_parser.add_argument('wf_file', help="path to a FireWork or Workflow file")
@@ -106,52 +110,55 @@ def lpad():
     rerun_fws_parser.add_argument('-n', '--name', help='name', default=None)
     rerun_fws_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     rerun_fws_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    rerun_fws_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    rerun_fws_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     reservation_parser = subparsers.add_parser('detect_unreserved', help='Find launches with stale reservations')
     reservation_parser.add_argument('--time', help='expiration time (seconds)',
                                     default=FWConfig().RESERVATION_EXPIRATION_SECS, type=int)
-    reservation_parser.add_argument('--mark', help='cancel bad reservations', action='store_true')
+    reservation_parser.add_argument('--rerun', help='cancel and rerun expired reservations', action='store_true')
 
-    fizzled_parser = subparsers.add_parser('detect_fizzled', help='Find launches that have FIZZLED')
+    fizzled_parser = subparsers.add_parser('detect_lostruns', help='Find launches that have FIZZLED')
     fizzled_parser.add_argument('--time', help='expiration time (seconds)', default=FWConfig().RUN_EXPIRATION_SECS,
                                 type=int)
-    fizzled_parser.add_argument('--mark', help='mark fizzled', action='store_true')
+    fizzled_parser.add_argument('--fizzle', help='mark lost runs as fizzled', action='store_true')
+    fizzled_parser.add_argument('--rerun', help='rerun lost runs', action='store_true')
+    fizzled_parser.add_argument('--max_runtime', help='max runtime, helpful for tracing down walltime kills (seconds)', default=None,
+                                type=int)
 
     defuse_parser = subparsers.add_parser('defuse', help='cancel (de-fuse) an entire Workflow')
     defuse_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
     defuse_parser.add_argument('-n', '--name', help='name', default=None)
     defuse_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     defuse_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    defuse_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    defuse_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     archive_parser = subparsers.add_parser('archive', help='archive an entire Workflow (irreversible)')
     archive_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
     archive_parser.add_argument('-n', '--name', help='name', default=None)
     archive_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     archive_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    archive_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    archive_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     reignite_parser = subparsers.add_parser('reignite', help='reignite (un-cancel) an entire Workflow')
     reignite_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
     reignite_parser.add_argument('-n', '--name', help='name', default=None)
     reignite_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     reignite_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    reignite_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    reignite_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     defuse_fw_parser = subparsers.add_parser('defuse_fws', help='cancel (de-fuse) a single FireWork')
     defuse_fw_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
     defuse_fw_parser.add_argument('-n', '--name', help='name', default=None)
     defuse_fw_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     defuse_fw_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    defuse_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    defuse_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     reignite_fw_parser = subparsers.add_parser('reignite_fws', help='reignite (un-cancel) a single FireWork')
     reignite_fw_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
     reignite_fw_parser.add_argument('-n', '--name', help='name', default=None)
     reignite_fw_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     reignite_fw_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    reignite_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    reignite_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     maintain_parser = subparsers.add_parser('maintain', help='Run database maintenance')
     maintain_parser.add_argument('--infinite', help='loop infinitely', action='store_true')
@@ -166,7 +173,15 @@ def lpad():
     refresh_parser.add_argument('-n', '--name', help='name', default=None)
     refresh_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
     refresh_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
-    refresh_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    refresh_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+
+    priority_parser = subparsers.add_parser('set_priority', help='modify the priority of one or more FireWorks')
+    priority_parser.add_argument('priority', help='get FW with this fw_id', default=None, type=int)
+    priority_parser.add_argument('-i', '--fw_id', help='fw id or comma separated list of fw ids', default=None)
+    priority_parser.add_argument('-n', '--name', help='name', default=None)
+    priority_parser.add_argument('-s', '--state', help='state ("ARCHIVED", "DEFUSED", "READY", "RESERVED", "FIZZLED", "RUNNING", "COMPLETED")', default=None)
+    priority_parser.add_argument('-q', '--query', help='query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')', default=None)
+    priority_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
 
     subparsers.add_parser('version', help='Print the version and location of FireWorks installation')
 
@@ -218,13 +233,19 @@ def lpad():
             lp = LaunchPad(logdir=args.logdir, strm_lvl=args.loglvl)
 
         if args.command == 'reset':
+            if not args.password:
+                if raw_input('Are you sure? This will RESET {} workflows and all data. (Y/N)'.format(lp.workflows.count()))[0].upper() == 'Y': args.password=datetime.datetime.now().strftime('%Y-%m-%d')
+                else:
+                    raise ValueError('Operation aborted by user.')
             lp.reset(args.password)
 
-        elif args.command == 'detect_fizzled':
-            print lp.detect_fizzled(args.time, args.mark)
+        elif args.command == 'detect_lostruns':
+            fl,ff = lp.detect_lostruns(expiration_secs=args.time, fizzle=args.fizzle, rerun=args.rerun, max_runtime=args.max_runtime)
+            lp.m_logger.debug('Detected {} FIZZLED launches: {}'.format(len(fl), fl))
+            lp.m_logger.info('Detected {} FIZZLED FWs: {}'.format(len(ff), ff))
 
         elif args.command == 'detect_unreserved':
-            print lp.detect_unreserved(args.time, args.mark)
+            print lp.detect_unreserved(expiration_secs=args.time, rerun=args.rerun)
 
         elif args.command == 'add':
             fwf = Workflow.from_file(args.wf_file)
@@ -345,44 +366,58 @@ def lpad():
             fw_ids = parse_helper(lp, args, wf_mode=True)
             for f in fw_ids:
                 lp.defuse_wf(f)
-                print f
+                lp.m_logger.debug('Processed fw_id: {}'.format(f))
+            lp.m_logger.info('Finished defusing {} FWs'.format(len(fw_ids)))
 
         elif args.command == 'archive':
             fw_ids = parse_helper(lp, args, wf_mode=True)
             for f in fw_ids:
                 lp.archive_wf(f)
-                print f
+                lp.m_logger.debug('Processed fw_id: {}'.format(f))
+            lp.m_logger.info('Finished archiving {} FWs'.format(len(fw_ids)))
 
         elif args.command == 'reignite':
             fw_ids = parse_helper(lp, args, wf_mode=True)
             for f in fw_ids:
                 lp.reignite_wf(f)
-                print f
+                lp.m_logger.debug('Processed Workflow with fw_id: {}'.format(f))
+            lp.m_logger.info('Finished reigniting {} Workflows'.format(len(fw_ids)))
 
         elif args.command == 'defuse_fws':
             fw_ids = parse_helper(lp, args)
             for f in fw_ids:
                 lp.defuse_fw(f)
-                print f
+                lp.m_logger.debug('Processed fw_id: {}'.format(f))
+            lp.m_logger.info('Finished defusing {} FWs'.format(len(fw_ids)))
 
         elif args.command == 'reignite_fws':
             fw_ids = parse_helper(lp, args)
             for f in fw_ids:
                 lp.reignite_fw(f)
-                print f
+                lp.m_logger.debug('Processed fw_id: {}'.format(f))
+            lp.m_logger.info('Finished reigniting {} FWs'.format(len(fw_ids)))
 
         elif args.command == 'rerun_fws':
             fw_ids = parse_helper(lp, args)
             for f in fw_ids:
                 lp.rerun_fw(int(f))
-                print f
+                lp.m_logger.debug('Processed fw_id: {}'.format(f))
+            lp.m_logger.info('Finished rerunning {} FWs'.format(len(fw_ids)))
 
         elif args.command == 'refresh':
             fw_ids = parse_helper(lp, args, wf_mode=True)
             for f in fw_ids:
                 wf = lp.get_wf_by_fw_id(f)
                 lp._refresh_wf(wf, f)
-                print f
+                lp.m_logger.debug('Processed Workflow with fw_id: {}'.format(f))
+            lp.m_logger.info('Finished refreshing {} Workflows'.format(len(fw_ids)))
+
+        elif args.command == 'set_priority':
+            fw_ids = parse_helper(lp, args, wf_mode=True)
+            for f in fw_ids:
+                lp.set_priority(f, args.priority)
+                lp.m_logger.debug("Processed fw_id {}".format(f))
+            lp.m_logger.info("Finished setting priorities of {} FWs".format(len(fw_ids)))
 
         elif args.command == 'webgui':
             os.environ.setdefault("DJANGO_SETTINGS_MODULE", "fireworks.base_site.settings")
@@ -425,7 +460,9 @@ def lpad():
             fw_ids = parse_helper(lp, args)
             for f in fw_ids:
                 lp.forget_offline(f)
-                print f
+                lp.m_logger.debug('Processed fw_id: {}'.format(f))
+
+            lp.m_logger.info('Finished forget_offine, processed {} FWs'.format(len(fw_ids)))
 
 
 if __name__ == '__main__':
